/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtTest 1.0
import Unity.Test 0.1 as UT
import ".."
import "../../../Launcher"
import "../../../Applications/applications.js" as ApplicationsModel

/* Nothing is shown at first. If you drag from left edge you will bring up the
   launcher. */
Item {
    width: units.gu(50)
    height: units.gu(81)

    Launcher {
        id: launcher
        x: 0
        y: 0
        width: units.gu(40)
        height: units.gu(71)

        favourites: ["/usr/share/applications/phone-app.desktop",
                     "/usr/share/applications/camera-app.desktop",
                     "/usr/share/applications/gallery-app.desktop"]

        shortcutsWidth: units.gu(9)
        shortcutsThreshold: units.gu(2)
        iconPath: "graphics/applicationIcons"
        __dateTime: testCase.fakeDateTime

        property string latestApplicationSelected

        onLauncherApplicationSelected: {
            latestApplicationSelected = name
        }

        property int dashItemSelected_count: 0
        onDashItemSelected: {
            dashItemSelected_count++;
        }
    }

    UT.UnityTestCase {
        id: testCase
        name: "Launcher"
        when: windowShown

        // Drag from the left edge of the screen rightwards and check that the launcher
        // appears (as if being dragged by the finger/pointer)
        function test_dragLeftEdgeToRevealLauncherAndTapCenterToDismiss() {
            // "shortcuts" is the vertical stripe of icons the user see.
            // i.e., the launcher graphical element itself.
            var shortcuts = findChild(launcher, "shortcuts")
            verify(shortcuts != undefined)

            // it starts out hidden just left of the left launcher edge
            compare(shortcuts.x, -shortcuts.width)

            dragLauncherIntoView()

            // tapping on the center of the screen should dismiss the launcher
            mouseClick(launcher, launcher.width/2, launcher.height/2)

            // should eventually get fully retracted (hidden)
            tryCompare(shortcuts, "x", -shortcuts.width, 1000)
        }

        /* If I click on the icon of an application on the launcher
           Launcher::launcherApplicationSelected signal should be emitted with the
           corresponding desktop file. E.g. clicking on phone icon should yield
           launcherApplicationSelected("[...]phone-app.desktop") */
        function test_clickingOnAppIconCausesSignalEmission() {
            launcher.latestApplicationSelected = ""

            dragLauncherIntoView()

            var phoneIcon = findChild(launcher,
                                      "delegate /usr/share/applications/phone-app.desktop")
            verify(phoneIcon != undefined)

            mouseClick(phoneIcon, phoneIcon.width/2, phoneIcon.height/2)

            tryCompare(launcher, "latestApplicationSelected",
                       "/usr/share/applications/phone-app.desktop")

            // Tapping on an application icon also dismisses the launcher
            waitUntilLauncherDisappears()
        }

        /* If I click on the dash icon on the launcher
           Launcher::dashItemSelected signal should be emitted */
        function test_clickingOnDashIconCausesSignalEmission() {
            launcher.dashItemSelected_count = 0

            dragLauncherIntoView()

            var dashIcon = findChild(launcher,
                                      "delegate dash.desktop")
            verify(dashIcon != undefined)

            mouseClick(dashIcon, dashIcon.width/2, dashIcon.height/2)

            tryCompare(launcher, "dashItemSelected_count", 1)

            // Tapping on the dash icon also dismisses the launcher
            waitUntilLauncherDisappears()
        }

        /* Check that if, while dragging, you put the pointer next to an app icon,
           a label/bubble (LauncherLabel) will be shown containing its name and icon.
           When the pointer is released it should activate the corresponding application  */
        function test_LauncherLabel() {
            launcher.latestApplicationSelected = ""
            var startX = launcher.shortcutsThreshold/2
            var startY = launcher.height/2
            var phoneDesktopFile = "/usr/share/applications/phone-app.desktop"

            mouseFlick(launcher,
                       startX, startY,
                       launcher.width/2, startY,
                       true, /* do press */
                       false, /* but do not release yet */
                       launcher.maxFlickSpeedToLaunchApp/2 /* speed */ )

            // "shortcuts" is the vertical stripe of icons the user see.
            // i.e., the launcher graphical element itself.
            var shortcuts = findChild(launcher, "shortcuts")
            verify(shortcuts != undefined)

            // wait until it gets fully extended
            tryCompare(shortcuts, "x", 0)

            var phoneIcon = findChild(launcher, "delegate " + phoneDesktopFile)
            verify(phoneIcon != undefined)

            var phoneIconLauncherPos = phoneIcon.mapToItem(launcher, 0, 0)
            // Move mouse to the side of the phone icon
            fakeDateTime.currentTimeMs += 600
            mouseMove(launcher, launcher.width/2, phoneIconLauncherPos.y + phoneIcon.height/2)

            var launcherLabel = findChild(launcher, "launcherLabel")
            verify(launcherLabel != undefined)

            var phoneAppInfo = ApplicationsModel.__availableApplications[phoneDesktopFile];
            tryCompare(launcherLabel, "text", phoneAppInfo["name"])
            tryCompare(launcherLabel, "opacity", 1)
            tryCompare(launcherLabel, "visible", true)
            wait(1000)

            mouseRelease(launcher, launcher.width/2, phoneIconLauncherPos.y + phoneIcon.height/2)

            tryCompare(launcher, "latestApplicationSelected",
                       "/usr/share/applications/phone-app.desktop")

            waitUntilLauncherDisappears()
        }

        /* If you flick too fast no application will be launched. You will
           just bring up the launcher. */
        function test_maxFlickSpeedToLaunchApp() {
            launcher.latestApplicationSelected = ""
            var phoneDesktopFile = "/usr/share/applications/phone-app.desktop"

            // "shortcutd" is the vertical stripe of icons the user see.
            // i.e., the launcher graphical element itself.
            var shortcuts = findChild(launcher, "shortcuts")
            verify(shortcuts != undefined)

            var phoneIcon = findChild(launcher, "delegate " + phoneDesktopFile)
            verify(phoneIcon != undefined)

            var phoneIconLauncherPos = phoneIcon.mapToItem(launcher, 0, 0)

            // flick through the area where the phone app icon is (or will be once the launcher
            // comes up)
            var startX = launcher.shortcutsThreshold/2
            var startY = phoneIconLauncherPos.y + phoneIcon.height/2
            mouseFlick(launcher,
                       startX, startY,
                       launcher.width/2, startY,
                       true, /* do press */
                       false, /* but do not release yet */
                       launcher.maxFlickSpeedToLaunchApp*2 /* speed */ )

            // wait until it gets fully extended
            tryCompare(shortcuts, "x", 0)

            // check that the phone icon indeed got highlighted
            compare(shortcuts.model.get(shortcuts.currentIndex).desktopFile,
                    phoneDesktopFile)

            fakeDateTime.currentTimeMs += 10
            mouseRelease(launcher, launcher.width/2, startY)

            // wait for a bit to ensure that the launcher is not going anywhere.
            wait(launcher.inactivityTimeout/6)
            compare(shortcuts.x, 0)

            // check that the phone appt didn't get activated in the end.
            // no selection was made
            compare(launcher.latestApplicationSelected, "")

            // now dismiss the launcher by tapping outside the "shortcuts" stripe,
            // on the middle of the screen
            mouseClick(launcher, launcher.width/2, launcher.height/2)

            waitUntilLauncherDisappears()
        }

        function dragLauncherIntoView() {
            var startX = launcher.shortcutsThreshold/2
            var startY = launcher.height/2
            mouseFlick(launcher,
                       startX, startY,
                       startX+units.gu(8), startY)

            // "shortcuts" is the vertical stripe of icons the user see.
            // i.e., the launcher graphical element itself.
            var shortcuts = findChild(launcher, "shortcuts")
            verify(shortcuts != undefined)

            // wait until it gets fully extended
            tryCompare(shortcuts, "x", 0)
        }

        function waitUntilLauncherDisappears() {
            var shortcuts = findChild(launcher, "shortcuts")
            tryCompare(shortcuts, "x", -shortcuts.width, 1000)
        }
    }
}
